"""
velvet datatypes
James E Johnson - University of Minnesota
for velvet assembler tool in galaxy
"""

import data
from galaxy.datatypes import sequence
import logging, os, sys, time, tempfile, shutil, string, glob, re
import galaxy.model
from galaxy.datatypes import metadata
from galaxy.datatypes.metadata import MetadataElement
from galaxy import util
from galaxy.datatypes.images import Html
from sniff import *

log = logging.getLogger(__name__)

class Sequences( sequence.Fasta ):
    """Class describing the Sequences file generated by velveth """

    def sniff( self, filename ):
        """
        Determines whether the file is a velveth produced  fasta format
        """

        try:
            fh = open( filename )
            while True:
                line = fh.readline()
                if not line:
                    break #EOF
                line = line.strip()
                if line: #first non-empty line
                    if line.startswith( '>' ):
                        if not re.match(r'>[^\t]+\t\d+\t\d+$',line):
                            break
                        #The next line.strip() must not be '', nor startwith '>'
                        line = fh.readline().strip()
                        if line == '' or line.startswith( '>' ):
                            break
                        return True
                    else:
                        break #we found a non-empty line, but its not a fasta header
            fh.close()
        except:
            pass
        return False

class Roadmaps( data.Text ):
    """Class describing the Sequences file generated by velveth """

    def sniff( self, filename ):
        """
        Determines whether the file is a velveth produced RoadMap
        """

        try:
            fh = open( filename )
            while True:
                line = fh.readline()
                if not line:
                    break #EOF
                line = line.strip()
                if line: #first non-empty line
                    if not re.match(r'\d+\t\d+\t\d+$',line):
                        break
                    #The next line.strip() should be 'ROADMAP 1'
                    line = fh.readline().strip()
                    if not re.match(r'ROADMAP \d+$',line):
                        break
                    return True
                else:
                    break #we found a non-empty line, but its not a fasta header
            fh.close()
        except:
            pass
        return False

class Velvet( Html ):
    MetadataElement( name="base_name", desc="base name for velveth dataset", default="velvet", readonly=True, set_in_upload=True)
    MetadataElement( name="paired_end_reads", desc="has paired-end reads", default="False", readonly=False, set_in_upload=True)
    MetadataElement( name="long_reads", desc="has long reads", default="False", readonly=False, set_in_upload=True)
    MetadataElement( name="short2_reads", desc="has 2nd short reads", default="False", readonly=False, set_in_upload=True)
    composite_type = 'auto_primary_file'
    allow_datatype_change = False
    file_ext = 'html'

    def __init__( self, **kwd ):
        Html.__init__( self, **kwd )
        self.add_composite_file( 'Sequences', mimetype = 'text/html', description = 'Sequences', substitute_name_with_metadata = None, is_binary = False )
        self.add_composite_file( 'Roadmaps', mimetype = 'text/html', description = 'Roadmaps', substitute_name_with_metadata = None, is_binary = False )
        self.add_composite_file( 'Log', mimetype = 'text/html', description = 'Log', substitute_name_with_metadata = None, is_binary = False )

    def generate_primary_file( self, dataset = None ):
        rval = ['<html><head><title>Velvet Galaxy Composite Dataset </title></head><p/>']
        rval.append('<div>This composite dataset is composed of the following files:<p/><ul>')
        for composite_name, composite_file in self.get_composite_files( dataset = dataset ).iteritems():
            fn = composite_name
            opt_text = ''
            if composite_file.optional:
                opt_text = ' (optional)'
            if composite_file.get('description'):
                rval.append( '<li><a href="%s" type="text/plain">%s (%s)</a>%s</li>' % ( fn, fn, composite_file.get('description'), opt_text ) )
            else:
                rval.append( '<li><a href="%s" type="text/plain">%s</a>%s</li>' % ( fn, fn, opt_text ) )
        rval.append( '</ul></div></html>' )
        return "\n".join( rval )

    def regenerate_primary_file(self,dataset):
        """
        cannot do this until we are setting metadata 
        """
        bn = dataset.metadata.base_name
        efp = dataset.extra_files_path
        flist = os.listdir(efp)
        log_path = os.path.join(efp,'Log')
        f = open(log_path,'r')
        log_content = f.read(1000)
        f.close()
        log_msg = re.sub('/\S*/','',log_content)
        paired_end_reads = re.search('-(short|long)Paired', log_msg) != None
        metadata.paired_end_reads = paired_end_reads
        long_reads = re.search('-long', log_msg) != None
        metadata.long_reads = long_reads
        short2_reads = re.search('-short(Paired)?2', log_msg) != None
        metadata.short2_reads = short2_reads
        dataset.info = re.sub('.*velveth \S+','hash_length',re.sub('\n',' ',log_msg))
        log.debug( "Velveth log info  %s" % log_msg)
        rval = ['<html><head><title>Velvet Galaxy Composite Dataset </title></head><p/>']
        rval.append('<div>Generated:<p/><code> %s </code></div>' %(re.sub('\n','<br>',log_msg)))
        rval.append('<div>Velveth dataset:<p/><ul>')
        for composite_name, composite_file in self.get_composite_files( dataset = dataset ).iteritems():
            fn = composite_name
            opt_text = ''
            if composite_file.optional:
                opt_text = ' (optional)'
            if composite_file.get('description'):
                rval.append( '<li><a href="%s" type="text/plain">%s (%s)</a>%s</li>' % ( fn, fn, composite_file.get('description'), opt_text ) )
            else:
                rval.append( '<li><a href="%s" type="text/plain">%s</a>%s</li>' % ( fn, fn, opt_text ) )
        rval.append( '</ul></div></html>' )
        f = file(dataset.file_name,'w')
        f.write("\n".join( rval ))
        f.write('\n')
        f.close()

    def set_meta( self, dataset, **kwd ):
        Html.set_meta( self, dataset, **kwd )
        self.regenerate_primary_file(dataset)

if __name__ == '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])

